@@ -1,4 +1,4 @@
-# Copyright (c) 2018-2023 Micro Focus or one of its affiliates.
+# Copyright (c) 2018-2023 Open Text.
 # Copyright (c) 2018 Uber Technologies, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -39,11 +39,13 @@ from __future__ import print_function, division, absolute_import
 import datetime
 import glob
 import inspect
+import os
 import re
 import sys
 import traceback
 from decimal import Decimal
 from io import IOBase, BytesIO, StringIO
+from math import isnan
 from tempfile import NamedTemporaryFile, SpooledTemporaryFile, TemporaryFile
 from uuid import UUID
 from collections import OrderedDict
@@ -227,9 +229,17 @@ class Cursor(object):
         use_prepared = bool(self.connection.options['use_prepared_statements']
                 if use_prepared_statements is None else use_prepared_statements)
         if use_prepared:
+            #################################################################
             # Execute the SQL as prepared statement (server-side bindings)
-            if parameters and not isinstance(parameters, (list, tuple)):
-                raise TypeError("Execute parameters should be a list/tuple")
+            #################################################################
+            if parameters is not None:
+                if not isinstance(parameters, (list, tuple)):
+                    raise TypeError("Execute parameters should be a list/tuple")
+                elif parameters and '?' not in operation:
+                    raise ValueError(f'Invalid SQL: {operation}'
+                        '\nHINT: When use_prepared_statements=True, variables in SQL should be specified with '
+                        'question mark (?) placeholders. Positional format (%s) placeholders have to be used '
+                        'with use_prepared_statements=False setting.')
 
             # If the SQL has not been prepared, prepare the SQL
             if operation != self.prepared_sql:
@@ -239,7 +249,9 @@ class Cursor(object):
             # Bind the parameters and execute
             self._execute_prepared_statement([parameters])
         else:
+            #################################################################
             # Execute the SQL directly (client-side bindings)
+            #################################################################
             if parameters:
                 operation = self.format_operation_with_parameters(operation, parameters)
             self._execute_simple_query(operation)
@@ -265,7 +277,9 @@ class Cursor(object):
                 if use_prepared_statements is None else use_prepared_statements)
 
         if use_prepared:
+            #################################################################
             # Execute the SQL as prepared statement (server-side bindings)
+            #################################################################
             if len(seq_of_parameters) == 0:
                 raise ValueError("seq_of_parameters should not be empty")
             if not all(isinstance(elem, (list, tuple)) for elem in seq_of_parameters):
@@ -278,6 +292,9 @@ class Cursor(object):
             # Bind the parameters and execute
             self._execute_prepared_statement(seq_of_parameters)
         else:
+            #################################################################
+            # Rewrite the INSERT SQL into a COPY statement
+            #################################################################
             m = self._insert_statement.match(operation)
             if m:
                 target = as_text(m.group('target'))
@@ -589,13 +606,39 @@ class Cursor(object):
             return str(py_obj)
         elif isinstance(py_obj, (str, bytes)):
             return self.format_quote(as_text(py_obj), is_copy_data)
-        elif isinstance(py_obj, (int, float, Decimal)):
+        elif isinstance(py_obj, (int, Decimal)):
+            return str(py_obj)
+        elif isinstance(py_obj, float):
+            if py_obj in (float('Inf'), float('-Inf')) or isnan(py_obj):
+                return f"'{str(py_obj)}'::FLOAT"
             return str(py_obj)
         elif isinstance(py_obj, tuple):  # tuple and namedtuple
             elements = [None] * len(py_obj)
             for i in range(len(py_obj)):
                 elements[i] = self.object_to_string(py_obj[i], is_copy_data)
             return "(" + ",".join(elements) + ")"
+        elif isinstance(py_obj, list) and not is_copy_data:
+            elements = [None] * len(py_obj)
+            for i in range(len(py_obj)):
+                elements[i] = self.object_to_string(py_obj[i], False)
+            # Use the ARRAY keyword to construct an array value
+            return f'ARRAY[{",".join(elements)}]'
+        elif isinstance(py_obj, set) and not is_copy_data:
+            elements = [None] * len(py_obj)
+            i = 0
+            for o in py_obj:
+                elements[i] = self.object_to_string(o, False)
+                i += 1
+            # Use the SET keyword to construct a set value
+            return f'SET[{",".join(elements)}]'
+        elif isinstance(py_obj, dict) and not is_copy_data:
+            elements = [None] * len(py_obj)
+            i = 0
+            for k, v in py_obj.items():
+                elements[i] = self.object_to_string(v, False) + f' AS "{k}"'
+                i += 1
+            # Use the ROW keyword to construct a row value
+            return f'ROW({",".join(elements)})'
         elif isinstance(py_obj, (datetime.datetime, datetime.date, datetime.time, UUID)):
             return self.format_quote(as_text(str(py_obj)), is_copy_data)
         else:
@@ -610,9 +653,11 @@ class Cursor(object):
 
     # noinspection PyArgumentList
     def format_operation_with_parameters(self, operation, parameters, is_copy_data=False):
-        operation = as_text(operation)
-
         if isinstance(parameters, dict):
+            if parameters and ':' not in operation and os.environ.get('VERTICA_PYTHON_IGNORE_NAMED_PARAMETER_CHECK') != '1':
+                raise ValueError(f'Invalid SQL: {operation}'
+                    "\nHINT: When argument 'parameters' is a dict, variables in SQL should be specified with named (:name) placeholders."
+                    " If you use a dict to represent the value of a ROW type column, enclose the dict with brackets('[]') to construct a list.")
             for key, param in parameters.items():
                 if not isinstance(key, str):
                     key = str(key)
@@ -626,11 +671,15 @@ class Cursor(object):
                 operation = re.sub(match_str, lambda _: value, operation, flags=re.U)
 
         elif isinstance(parameters, (tuple, list)):
+            if parameters and '%s' not in operation:
+                raise ValueError(f'Invalid SQL: {operation}'
+                    "\nHINT: When argument 'parameters' is a tuple/list, "
+                    'variables in SQL should be specified with positional format (%s) placeholders. '
+                    'Question mark (?) placeholders have to be used with use_prepared_statements=True setting.')
             tlist = []
             for param in parameters:
                 value = self.object_to_string(param, is_copy_data)
                 tlist.append(value)
-
             operation = operation % tuple(tlist)
         else:
             raise TypeError("Argument 'parameters' must be dict or tuple/list")
